#!/bin/sh

UBOOTIMG=/usr/local/share/u-boot/1100/flash-image.bin
FLASHDEV=/dev/flash/spi0

if [ ! -c ${FLASHDEV} -o ! -f ${UBOOTIMG} ]; then
	echo "Cannot update the Netgate 1100 U-boot.  Invalid settings."
	exit 1
fi

unset FORCE
if [ $# -gt 0 ]; then
	if [ "$1" == "-f" ]; then
		FORCE=1
	fi
fi

unset HAS_VALID_UBOOT
unset HAS_NEW_UBOOT
RETRY=5
while [ $RETRY -gt 0 ]; do
	UBOOT_SHA=$(/bin/dd if=/dev/flash/spi0 bs=64k count=1 2> /dev/null | /sbin/sha256)
	for SHA in $(/bin/cat /usr/local/share/u-boot/1100/u-boot-hash-all); do
		if [ "${UBOOT_SHA}" == "${SHA}" ]; then
			HAS_VALID_UBOOT=1
			break
		fi
	done
	if [ -n "${HAS_VALID_UBOOT}" ]; then
		break
	fi
	RETRY=$(( $RETRY - 1 ))
	sleep .5
done
if [ -z "${HAS_VALID_UBOOT}" -a $RETRY -eq 0 ]; then
	#
	# If it is not possible to identify the installed U-Boot
	# version, set HAS_NEW_UBOOT to skip the U-Boot update later
	# in the process.
	#
	HAS_NEW_UBOOT=1
fi
for SHA in $(/bin/cat /usr/local/share/u-boot/1100/u-boot-hash); do
	if [ "${UBOOT_SHA}" == "${SHA}" ]; then
		HAS_NEW_UBOOT=1
		break
	fi
done

if [ -n "${HAS_NEW_UBOOT}" ]; then
	echo -n "=> U-Boot is already at the latest version.  "
	if [ -z "${FORCE}" ]; then
		echo "Skipping..."
		exit 0
	else
		echo "Continuing with the installation..."
	fi
fi

echo "=> Updating the Netgate 1100 U-boot"
echo "==> Reading current settings"
RETRY=5
unset ENVOK
/bin/rm -f /tmp/ubenv 2> /dev/null
while [ $RETRY -gt 0 ]; do
	/usr/local/sbin/u-boot-env-read /tmp/ubenv 2> /dev/null
	if [ $? -eq 0 -a -f /tmp/ubenv ]; then
		ENVOK=1
		break;
	fi
	RETRY=$(( $RETRY - 1 ))
	sleep .5
done
if [ -z "${ENVOK}" ]; then
	echo
	echo "==> Warning: No settings were found"
	echo
	echo "==> WARNING: Skipping the U-Boot update!!!"
	echo
else
	echo "==> Updating the U-boot image (this may take a few minutes)"
	/bin/dd if=${UBOOTIMG} of=${FLASHDEV} bs=64k
	echo "==> Updating settings"
	/usr/local/sbin/u-boot-env-update /tmp/ubenv /tmp/newubenv 2> /dev/null
	if [ ${?} -ne 0 -o ! -f /tmp/newubenv ]; then
		echo "Failed to restore the settings, U-boot will not work properly."
		echo "Saving original settings in /root/ubenv for later recovery."
		cp /tmp/ubenv /root/ubenv
	fi
	echo "==> Restoring settings"
	/usr/local/sbin/u-boot-env-write -yY /tmp/newubenv 2> /dev/null
	if [ ${?} -ne 0 ]; then
		echo "Failed to restore the settings, U-boot will not work properly."
		echo "Saving original settings in /root/ubenv for later recovery."
		cp /tmp/ubenv /root/ubenv
	fi
	/bin/rm /tmp/ubenv /tmp/newubenv 2> /dev/null
fi
